{******************************************************************************}
{                                                                              }
{ HID API exerciser program                                                    }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is HidInfo.pas.                                            }
{                                                                              }
{ The Initial Developer of the Original Code is Robert Marquardt.              }
{ (robert_marquardt@gmx.de)                                                    }
{                                                                              }
{ Portions created by Robert Marquardt are                                     }
{ Copyright (c) 2003 Robert Marquardt.                                         }
{                                                                              }
{ Last modified: December 30, 2002                                             }
{                                                                              }
{******************************************************************************}
{                                                                              }
{ This program is a present to Jan Axelson, Lakeview Research.                 }
{ It is only available from Jan Axelson.                                       }
{                                                                              }
{******************************************************************************}

unit HidInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  JvHidControllerClass, ComCtrls;

type
  TInfoForm = class(TForm)
    DevStrings: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Vid: TLabel;
    Pid: TLabel;
    Vers: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    InputLen: TLabel;
    OutputLen: TLabel;
    FeatureLen: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    VendorName: TLabel;
    ProductName: TLabel;
    Label11: TLabel;
    SerialNo: TLabel;
    LangStrings: TListBox;
    Label12: TLabel;
    CollectionTree: TTreeView;
    Label13: TLabel;
    procedure FormShow(Sender: TObject);
  public
    TestDev: TJvHidDevice;
    procedure EnumerateNodes(Parent: TTreeNode; Idx, Count: WORD);
    procedure FillCollectionTree;
  end;

var
  InfoForm: TInfoForm;

implementation

uses
  HidUsage;

{$R *.DFM}

procedure TInfoForm.FormShow(Sender: TObject);
var
  I: Integer;
begin
  VendorName.Caption  := TestDev.VendorName;
  ProductName.Caption := TestDev.ProductName;
  SerialNo.Caption    := TestDev.SerialNumber;
  Vid.Caption         := IntToHex(TestDev.Attributes.VendorID, 4);
  Pid.Caption         := IntToHex(TestDev.Attributes.ProductID, 4);
  Vers.Caption        := IntToHex(TestDev.Attributes.VersionNumber, 4);
  InputLen.Caption    := Format('%u', [TestDev.Caps.InputReportByteLength]);
  OutputLen.Caption   := Format('%u', [TestDev.Caps.OutputReportByteLength]);
  FeatureLen.Caption  := Format('%u', [TestDev.Caps.FeatureReportByteLength]);
  for I := 1 to 255 do
    if TestDev.DeviceStrings[I] <> '' then
      DevStrings.Items.Add(Format('%3d) %s',[I+1,TestDev.DeviceStrings[I]]));
  for I := 0 to TestDev.LanguageStrings.Count - 1 do
    LangStrings.Items.Add(TestDev.LanguageStrings[I]);
  FillCollectionTree;
end;

procedure TInfoForm.EnumerateNodes(Parent: TTreeNode; Idx, Count: WORD);
var
  I: Integer;
  Node: TTreeNode;
  UsagePageText: string;
  UsageText: string;
  CollectionTypeText: string;
begin
  // add a list of sibling nodes to the device tree node Parent
  I := 0;
  repeat
    UsagePageText := '';
    UsageText := '';
    case TestDev.LinkCollectionNodes[Idx].LinkUsagePage of
      HID_USAGE_PAGE_GENERIC:
        begin
          UsagePageText := 'Generic Desktop';
          case TestDev.LinkCollectionNodes[Idx].LinkUsage of
            HID_USAGE_GENERIC_POINTER:
              UsageText := 'Pointing Device';
            HID_USAGE_GENERIC_MOUSE:
              UsageText := 'Mouse';
            HID_USAGE_GENERIC_JOYSTICK:
              UsageText := 'Joystick';
            HID_USAGE_GENERIC_GAMEPAD:
              UsageText := 'Gamepad';
            HID_USAGE_GENERIC_KEYBOARD:
              UsageText := 'Keyboard';
            HID_USAGE_GENERIC_KEYPAD:
              UsageText := 'Keypad';
            HID_USAGE_GENERIC_MULTIAXIS:
              UsageText := 'Multiaxis';
            HID_USAGE_GENERIC_SYSTEM_CTL:
              UsageText := 'System Control';
            HID_USAGE_GENERIC_X:
              UsageText := 'X Axis';
            HID_USAGE_GENERIC_Y:
              UsageText := 'Y Axis';
            HID_USAGE_GENERIC_Z:
              UsageText := 'Z Axis';
            HID_USAGE_GENERIC_RX:
              UsageText := 'Relative X Axis';
            HID_USAGE_GENERIC_RY:
              UsageText := 'Relative Y Axis';
            HID_USAGE_GENERIC_RZ:
              UsageText := 'Relative Z Axis';
            HID_USAGE_GENERIC_SLIDER:
              UsageText := 'Slider';
            HID_USAGE_GENERIC_DIAL:
              UsageText := 'Dial';
            HID_USAGE_GENERIC_WHEEL:
              UsageText := 'Wheel';
            HID_USAGE_GENERIC_HATSWITCH:
              UsageText := '';
            HID_USAGE_GENERIC_COUNTED_BUFFER:
              UsageText := 'Counted Buffer';
            HID_USAGE_GENERIC_BYTE_COUNT:
              UsageText := 'Byte Count';
            HID_USAGE_GENERIC_MOTION_WAKEUP:
              UsageText := 'Motion Wakeup';
            HID_USAGE_GENERIC_START:
              UsageText := 'Start';
            HID_USAGE_GENERIC_SELECT:
              UsageText := 'Select';
            HID_USAGE_GENERIC_VX:
              UsageText := 'Velocity X';
            HID_USAGE_GENERIC_VY:
              UsageText := 'Velocity Y';
            HID_USAGE_GENERIC_VZ:
              UsageText := 'Velocity Z';
            HID_USAGE_GENERIC_VBRX:
              UsageText := 'Velocity Brake X';
            HID_USAGE_GENERIC_VBRY:
              UsageText := 'Velocity Brake Y';
            HID_USAGE_GENERIC_VBRZ:
              UsageText := 'Velocity Brake Z';
            HID_USAGE_GENERIC_VNO:
              UsageText := 'VNO';
            HID_USAGE_GENERIC_SYSCTL_POWER:
              UsageText := 'System Control Power';
            HID_USAGE_GENERIC_SYSCTL_SLEEP:
              UsageText := 'System Control Sleep';
            HID_USAGE_GENERIC_SYSCTL_WAKE:
              UsageText := 'System Control Wake';
            HID_USAGE_GENERIC_SYSCTL_CONTEXT_MENU:
              UsageText := 'System Control Context Menu';
            HID_USAGE_GENERIC_SYSCTL_MAIN_MENU:
              UsageText := 'System Control Main Menu';
            HID_USAGE_GENERIC_SYSCTL_APP_MENU:
              UsageText := 'System Control App Menu';
            HID_USAGE_GENERIC_SYSCTL_HELP_MENU:
              UsageText := 'System Control Help Menu';
            HID_USAGE_GENERIC_SYSCTL_MENU_EXIT:
              UsageText := 'System Control Menu Exit';
            HID_USAGE_GENERIC_SYSCTL_MENU_SELECT:
              UsageText := 'System Control Menu Select';
            HID_USAGE_GENERIC_SYSCTL_MENU_RIGHT:
              UsageText := 'System Control Menu Right';
            HID_USAGE_GENERIC_SYSCTL_MENU_LEFT:
              UsageText := 'System Control Menu Left';
            HID_USAGE_GENERIC_SYSCTL_MENU_UP:
              UsageText := 'System Control Menu Up';
            HID_USAGE_GENERIC_SYSCTL_MENU_DOWN:
              UsageText := 'System Control Menu Down';
            HID_USAGE_GENERIC_SYSCTL_DPAD_UP:
              UsageText := 'System Control DPad Up';
            HID_USAGE_GENERIC_SYSCTL_DPAD_DOWN:
              UsageText := 'System Control DPad Down';
            HID_USAGE_GENERIC_SYSCTL_DPAD_RIGHT:
              UsageText := 'System Control DPad Right';
            HID_USAGE_GENERIC_SYSCTL_DPAD_LEFT:
              UsageText := 'System Control DPad Left';
          end;
        end;
      HID_USAGE_PAGE_SIMULATION:
        begin
          UsagePageText := 'Simulation';
          case TestDev.LinkCollectionNodes[Idx].LinkUsage of
            HID_USAGE_SIMULATION_RUDDER:
              UsageText := 'Rudder';
            HID_USAGE_SIMULATION_THROTTLE:
              UsageText := 'Throttle';
          end;
        end;
      HID_USAGE_PAGE_VR:
        UsagePageText := 'Virtual Reality';
      HID_USAGE_PAGE_SPORT:
        UsagePageText := 'Sport';
      HID_USAGE_PAGE_GAME:
        UsagePageText := 'Game';
      HID_USAGE_PAGE_KEYBOARD:
        UsagePageText := 'Keyboard';
      HID_USAGE_PAGE_LED:
        UsagePageText := 'LED';
      HID_USAGE_PAGE_BUTTON:
        UsagePageText := 'Button';
      HID_USAGE_PAGE_ORDINAL:
        UsagePageText := 'Ordinal';
      HID_USAGE_PAGE_TELEPHONY:
        begin
          UsagePageText := 'Telephony';
          case TestDev.LinkCollectionNodes[Idx].LinkUsage of
            HID_USAGE_TELEPHONY_PHONE:
              UsageText := 'Phone';
            HID_USAGE_TELEPHONY_ANSWERING_MACHINE:
              UsageText := 'Answering Machine';
            HID_USAGE_TELEPHONY_MESSAGE_CONTROLS:
              UsageText := 'Message Controls';
            HID_USAGE_TELEPHONY_HANDSET:
              UsageText := 'Handset';
            HID_USAGE_TELEPHONY_HEADSET:
              UsageText := 'Headset';
            HID_USAGE_TELEPHONY_KEYPAD:
              UsageText := 'Keypad';
            HID_USAGE_TELEPHONY_PROGRAMMABLE_BUTTON:
              UsageText := 'Programmable Button';
          end;
        end;
      HID_USAGE_PAGE_CONSUMER:
        UsagePageText := 'Consumer';
      HID_USAGE_PAGE_DIGITIZER:
        UsagePageText := 'Digitizer';
      HID_USAGE_PAGE_UNICODE:
        UsagePageText := 'Unicode';
      HID_USAGE_PAGE_ALPHANUMERIC:
        UsagePageText := 'Alphanumeric';
    end;

    if UsagePageText = '' then
      UsagePageText := Format('UsagePage=%u',
        [TestDev.LinkCollectionNodes[Idx].LinkUsagePage]);
    if UsageText = '' then
      UsageText := Format('Usage=%u',
        [TestDev.LinkCollectionNodes[Idx].LinkUsage]);

    case TestDev.LinkCollectionNodes[Idx].CollectionType of
      $00:
        CollectionTypeText := 'Physical';
      $01:
        CollectionTypeText := 'Application';
      $02:
        CollectionTypeText := 'Logical';
      $03:
        CollectionTypeText := 'Report';
      $04:
        CollectionTypeText := 'Named Array';
      $05:
        CollectionTypeText := 'Usage Switch';
      $06:
        CollectionTypeText := 'Usage Modifier';
      $07..$7F:
        CollectionTypeText := Format('Reserved $%.2x',
          [Cardinal(TestDev.LinkCollectionNodes[Idx].CollectionType)]);
      $80..$FF:
        CollectionTypeText := Format('Vendor-defined $%.2x',
          [Cardinal(TestDev.LinkCollectionNodes[Idx].CollectionType)]);
    end;

    Node := CollectionTree.Items.AddChild(Parent,
      UsagePageText + ': ' + UsageText + ' (' + CollectionTypeText + ')');

    // recurse to the children nodes
    if TestDev.LinkCollectionNodes[Idx].FirstChild <> 0 then
      EnumerateNodes(Node, Idx + TestDev.LinkCollectionNodes[Idx].FirstChild,
        TestDev.LinkCollectionNodes[Idx].NumberOfChildren);
    // follow the link to the next sibling
    Idx := TestDev.LinkCollectionNodes[Idx].NextSibling;
    Inc(I);
  until I >= Count;
end;

procedure TInfoForm.FillCollectionTree;
var
  Node: TTreeNode;
  Name: string;
begin
  Name := TestDev.ProductName;
  if Name = '' then
    Name := Format('VID=%.4x PID=%.4x',
      [TestDev.Attributes.VendorID, TestDev.Attributes.ProductID]);
  Node := CollectionTree.Items.Add(nil, Name);
  EnumerateNodes(Node, 1, TestDev.LinkCollectionNodes[1].NumberOfChildren);
  CollectionTree.FullExpand;
end;

end.
