{******************************************************************************}
{                                                                              }
{ HID API exerciser program                                                    }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is HidViewMain.pas.                                        }
{                                                                              }
{ The Initial Developer of the Original Code is Robert Marquardt.              }
{ (robert_marquardt@gmx.de)                                                    }
{                                                                              }
{ Portions created by Robert Marquardt are                                     }
{ Copyright (c) 2003 Robert Marquardt.                                         }
{                                                                              }
{ Last modified: December 30, 2002                                             }
{                                                                              }
{******************************************************************************}
{                                                                              }
{ This program is a present to Jan Axelson, Lakeview Research.                 }
{ It is only available from Jan Axelson.                                       }
{                                                                              }
{******************************************************************************}

unit HidViewMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, ComCtrls,
  JvHidControllerClass;

type
  THidViewMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Quit1: TMenuItem;
    DeviceListBox: TListBox;
    Label1: TLabel;
    HidCtl: TJvHidDeviceController;
    Test: TButton;
    Info: TButton;
    XTest: TButton;
    Label2: TLabel;
    Label3: TLabel;
    About: TButton;
    procedure Quit1Click(Sender: TObject);
    procedure HidCtlDeviceChange(Sender: TObject);
    function HidCtlEnumerate(const HidDev: TJvHidDevice;
      const Idx: Integer): Boolean;
    procedure TestClick(Sender: TObject);
    procedure XTestClick(Sender: TObject);
    procedure InfoClick(Sender: TObject);
    procedure AboutClick(Sender: TObject);
  private
  public
    DevList: TList;
  end;

var
  HidViewMainForm: THidViewMainForm;

implementation

uses
  HidTester, HidXTester, HidInfo, About;

{$R *.dfm}

procedure THidViewMainForm.Quit1Click(Sender: TObject);
begin
  Close;
end;

procedure THidViewMainForm.HidCtlDeviceChange(Sender: TObject);
var
  I: Integer;
  Dev: TJvHidDevice;
begin
  if Assigned(DeviceListBox) then
  begin
    if Assigned(DevList) then
    begin
      for I := 0 to DevList.Count-1 do
      begin
        Dev := DevList.Items[I];
        Dev.Free;
      end;
      DevList.Clear;
    end
    else
      DevList := TList.Create;
    DeviceListBox.Clear;
    HidCtl.Enumerate;
    if DeviceListBox.Items.Count > 0 then
      DeviceListBox.ItemIndex := 0;
  end;
end;

function THidViewMainForm.HidCtlEnumerate(const HidDev: TJvHidDevice;
  const Idx: Integer): Boolean;
var
  Dev: TJvHidDevice;
begin
  if Assigned(DeviceListBox) then
  begin
    if HidDev.ProductName <> '' then
      DeviceListBox.Items.Add(HidDev.ProductName)
    else
      DeviceListBox.Items.Add(Format('VID=%.4x PID=%.4x Version=%.4x',
        [HidDev.Attributes.VendorID, HidDev.Attributes.ProductID, HidDev.Attributes.VersionNumber]));
    HidCtl.CheckOutByIndex(Dev, Idx);
    DevList.Add(Dev);
  end;
  Result := True;
end;

procedure THidViewMainForm.TestClick(Sender: TObject);
begin
  Application.HintPause := 100;
  Application.HintHidePause := 10000;
  if DeviceListBox.ItemIndex >= 0 then
    with TBasicTestForm.Create(Self) do
      try
        TestDev := DevList.Items[DeviceListBox.ItemIndex];
        Caption := Caption + ' "' + DeviceListBox.Items[DeviceListBox.ItemIndex] + '"';
        ShowModal;
      finally
        Free;
      end;
end;

procedure THidViewMainForm.XTestClick(Sender: TObject);
begin
  Application.HintPause := 100;
  Application.HintHidePause := 10000;
  if DeviceListBox.ItemIndex >= 0 then
    with TExtendedTestForm.Create(Self) do
      try
        TestDev := DevList.Items[DeviceListBox.ItemIndex];
        Caption := Caption + ' "' + DeviceListBox.Items[DeviceListBox.ItemIndex] + '"';
        ShowModal;
      finally
        Free;
      end;
end;

procedure THidViewMainForm.InfoClick(Sender: TObject);
begin
  Application.HintPause := 100;
  Application.HintHidePause := 10000;
  if DeviceListBox.ItemIndex >= 0 then
    with TInfoForm.Create(Self) do
      try
        TestDev := DevList.Items[DeviceListBox.ItemIndex];
        Caption := Caption + ' "' + DeviceListBox.Items[DeviceListBox.ItemIndex] + '"';
        ShowModal;
      finally
        Free;
      end;
end;

procedure THidViewMainForm.AboutClick(Sender: TObject);
begin
  with TAboutForm.Create(Self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

end.
