{******************************************************************************}
{                                                                              }
{ HID API exerciser program                                                    }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is HidXTester.pas.                                         }
{                                                                              }
{ The Initial Developer of the Original Code is Robert Marquardt.              }
{ (robert_marquardt@gmx.de)                                                    }
{                                                                              }
{ Portions created by Robert Marquardt are                                     }
{ Copyright (c) 2003 Robert Marquardt.                                         }
{                                                                              }
{ Last modified: December 30, 2002                                             }
{                                                                              }
{******************************************************************************}
{                                                                              }
{ This program is a present to Jan Axelson, Lakeview Research.                 }
{ It is only available from Jan Axelson.                                       }
{                                                                              }
{******************************************************************************}

unit HidXTester;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  JvHidControllerClass, Hid;

type
  TExtendedTestForm = class(TForm)
    ConfigurationBtn: TButton;
    ConfigurationLbl: TLabel;
    LinkCollectionNodesLbl: TLabel;
    LinkCollectionNodesBtn: TButton;
    ManufacturerStringLbl: TLabel;
    ManufacturerStringBtn: TButton;
    ProductStringLbl: TLabel;
    ProductStringBtn: TButton;
    SerialNumberStringLbl: TLabel;
    SerialNumberStringBtn: TButton;
    PhysicalDescriptorLbl: TLabel;
    PhysicalDescriptorBtn: TButton;
    HIDLanguagesLbl: TLabel;
    HIDLanguagesBtn: TButton;
    HIDStringsLbl: TLabel;
    HIDStringsBtn: TButton;
    Label1: TLabel;
    procedure ConfigurationBtnClick(Sender: TObject);
    procedure LinkCollectionNodesBtnClick(Sender: TObject);
    procedure ManufacturerStringBtnClick(Sender: TObject);
    procedure ProductStringBtnClick(Sender: TObject);
    procedure SerialNumberStringBtnClick(Sender: TObject);
    procedure PhysicalDescriptorBtnClick(Sender: TObject);
    procedure HIDLanguagesBtnClick(Sender: TObject);
    procedure HIDStringsBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  public
    TestDev: TJvHidDevice;
    PrepData: PHIDPPreparsedData;
    Config: THIDDConfiguration;
    Caps: THIDPCaps;
    CapsOK: Boolean;
    LinkCollection: array of THIDPLinkCollectionNode;
    PhysicalDescriptor: TJvPhysicalDescriptor;
  end;

var
  TestForm: TExtendedTestForm;

implementation

{$R *.dfm}

procedure TExtendedTestForm.ConfigurationBtnClick(Sender: TObject);
var
  I: Integer;
  Err: NTSTATUS;
begin
  ConfigurationLbl.Caption := '';
  Config.cookie         := nil;
  Config.size           := 0;
  Config.RingBufferSize := 0;

  if TestDev.OpenFile then
  begin
    for I := 0 to SizeOf(THIDDConfiguration)-1 do
      if HidD_GetConfiguration(TestDev.HidFileHandle, Config, I) then
      begin
        ConfigurationLbl.Caption := Format('tells success for insufficient buffer size %d', [I]);
        Break;
      end
      else
      begin
        Err := GetLastError;
        if not (((I <= 3) and (Err = ERROR_NOACCESS)) or ((I >= 4)  and (Err = ERROR_INVALID_FUNCTION))) then
        begin
          ConfigurationLbl.Caption := Format('failed with wrong error at buffer size %d (%s)', [I, SysErrorMessage(Err)]);
          Break;
        end;
      end;
    if ConfigurationLbl.Caption = '' then
    begin
      if HidD_GetConfiguration(TestDev.HidFileHandle, Config, SizeOf(THIDDConfiguration)) then
        ConfigurationLbl.Caption := 'succeeded'
      else
      if GetLastError = ERROR_INVALID_FUNCTION then
        ConfigurationLbl.Caption := 'failed because of no configurations'
      else
        ConfigurationLbl.Caption := Format('failed unexpectedly (%s)', [SysErrorMessage(GetLastError)]);
    end;
  end
  else
    ConfigurationLbl.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.LinkCollectionNodesBtnClick(Sender: TObject);
var
  Siz: ULONG;
  Ret: NTSTATUS;
  I: Integer;
begin
  LinkCollectionNodesLbl.Caption := '';
  if PrepData = nil then
    LinkCollectionNodesLbl.Caption := 'Preparsed data not available to call HidP_GetLinkCollectionNodes'
  else
  if not CapsOK then
    LinkCollectionNodesLbl.Caption := 'Caps not available to call HidP_GetLinkCollectionNodes'
  else
  begin
    Siz := Caps.NumberLinkCollectionNodes;
    SetLength(LinkCollection, Siz);
    for I := 0 to Siz-1 do
    begin
      Siz := I;
      Ret := HidP_GetLinkCollectionNodes(@LinkCollection[0], Siz, PrepData);
      if Ret = HIDP_STATUS_SUCCESS then
      begin
        LinkCollectionNodesLbl.Caption := Format('tells success for insufficient buffer element count %d', [I]);
        Break;
      end
      else
      if Ret <> HIDP_STATUS_BUFFER_TOO_SMALL then
      begin
        LinkCollectionNodesLbl.Caption := Format('failed with wrong error (%.8x)', [Ret]);
        Break;
      end;
    end;
    if LinkCollectionNodesLbl.Caption = '' then
    begin
      Siz := Caps.NumberLinkCollectionNodes;
      Ret := HidP_GetLinkCollectionNodes(@LinkCollection[0], Siz, PrepData);
      if Ret = HIDP_STATUS_SUCCESS then
        LinkCollectionNodesLbl.Caption := Format('successfully called (%u nodes)', [Siz])
      else
        LinkCollectionNodesLbl.Caption := Format('failed unexpectedly (%u nodes expected %u nodes reported)',
          [Caps.NumberLinkCollectionNodes, Siz]);
    end;
  end;
end;

procedure TExtendedTestForm.ManufacturerStringBtnClick(Sender: TObject);
var
  I: Integer;
  Err: NTSTATUS;
  Buffer: array [0..253] of WideChar;
begin
  ManufacturerStringLbl.Caption := '';
  if TestDev.OpenFile then
  begin
    for I := 0 to SizeOf(Buffer) do
    begin
      FillChar(Buffer, SizeOf(Buffer), #0);
      if HidD_GetManufacturerString(TestDev.HidFileHandle, Buffer, I) then
      begin
        ManufacturerStringLbl.Caption := Format('successfully called ("%s")', [Buffer]);
        Break;
      end
      else
      begin
        Err := GetLastError;
        if Err <> ERROR_INVALID_USER_BUFFER then
        begin
          ManufacturerStringLbl.Caption := Format('failed (%s)', [SysErrorMessage(Err)]);
          Break;
        end;
      end;
    end;
    if ManufacturerStringLbl.Caption = '' then
      ManufacturerStringLbl.Caption := 'failed to retrieve manufacturer string';
  end
  else
    ManufacturerStringLbl.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.ProductStringBtnClick(Sender: TObject);
var
  I: Integer;
  Err: NTSTATUS;
  Buffer: array [0..253] of WideChar;
begin
  ProductStringLbl.Caption := '';
  if TestDev.OpenFile then
  begin
    for I := 0 to SizeOf(Buffer) do
    begin
      FillChar(Buffer, SizeOf(Buffer), #0);
      if HidD_GetProductString(TestDev.HidFileHandle, Buffer, I) then
      begin
        ProductStringLbl.Caption := Format('successfully called ("%s")', [Buffer]);
        Break;
      end
      else
      begin
        Err := GetLastError;
        if Err <> ERROR_INVALID_USER_BUFFER then
        begin
          ProductStringLbl.Caption := Format('failed (%s)', [SysErrorMessage(Err)]);
          Break;
        end;
      end;
    end;
    if ProductStringLbl.Caption = '' then
      ProductStringLbl.Caption := 'failed to retrieve product string';
  end
  else
    ProductStringLbl.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.SerialNumberStringBtnClick(Sender: TObject);
var
  Err: NTSTATUS;
  I: Integer;
  J: Integer;
  Len: Integer;
  IDs:    array [0..253] of WORD;
  Buffer: array [0..253] of WideChar;
  SerialNumber: string;
begin
  SerialNumberStringLbl.Caption := '';
  if TestDev.OpenFile then
  begin
    for I := 0 to SizeOf(Buffer) do
    begin
      FillChar(Buffer, SizeOf(Buffer), #0);
      if HidD_GetSerialNumberString(TestDev.HidFileHandle, Buffer, I) then
      begin
        // calculate length of StringDescriptor 0
        FillChar(IDs, SizeOf(IDs), $FF);
        Len := 0;
        HidD_GetIndexedString(TestDev.HidFileHandle, 0, PWideChar(@IDs), SizeOf(IDs));
        for J := High(IDs) downto 0 do
          if IDs[J] <> $FFFF then
          begin
            if IDs[J] = 0 then
              Len := J
            else
              Len := J+1;
            Break;
          end;
        SerialNumber := '';
        // compensate for buggy function
        for J := 0 to Len - 1 do
          if IDs[J] <> WORD(Buffer[J]) then
          begin
            SerialNumber := Buffer;
            Break;
          end;
        SerialNumberStringLbl.Caption := Format('successfully called ("%s")', [SerialNumber]);
        Break;
      end
      else
      begin
        Err := GetLastError;
        if Err <> ERROR_INVALID_USER_BUFFER then
        begin
          SerialNumberStringLbl.Caption := Format('failed (%s)', [SysErrorMessage(Err)]);
          Break;
        end;
      end;
    end;
    if SerialNumberStringLbl.Caption = '' then
      SerialNumberStringLbl.Caption := 'failed to retrieve serial number string';
  end
  else
    SerialNumberStringBtn.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.PhysicalDescriptorBtnClick(Sender: TObject);
var
  I: Integer;
begin
  PhysicalDescriptorBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    I := 0;
    SetLength(PhysicalDescriptor, 2048);
    while not HidD_GetPhysicalDescriptor(TestDev.HidFileHandle, PhysicalDescriptor[0], I*SizeOf(WORD)) do
    begin
      Inc(I);
      if (I > 2048) or (GetLastError <> ERROR_INVALID_USER_BUFFER) then
      begin
        I := 0;
        PhysicalDescriptorLbl.Caption := Format('failed (%s)', [SysErrorMessage(GetLastError)]);
        Break;
      end;
    end;
    if I > 0 then
    begin
      if (I = 1) and (PhysicalDescriptor[0] = 0) then
        PhysicalDescriptorLbl.Caption :=  'succeeded (empty descriptor)'
      else
        PhysicalDescriptorLbl.Caption := Format('succeeded (Length=%d)', [I]);
      PhysicalDescriptorBtn.Tag := 2;
    end;
    SetLength(PhysicalDescriptor, I);
  end
  else
    PhysicalDescriptorLbl.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.HIDLanguagesBtnClick(Sender: TObject);
var
  I:   Integer;
  J:   Integer;
  Len: Integer;
  IDs: array [0..253] of WORD;
begin
  if TestDev.OpenFile then
  begin
    for J := 0 to SizeOf(IDs) do
    begin
      FillChar(IDs, SizeOf(IDs), $FF);
      Len := 0;
      if HidD_GetIndexedString(TestDev.HidFileHandle, 0, PWideChar(@IDs), J) then
      begin
        for I := High(IDs) downto 0 do
          if IDs[I] <> $FFFF then
          begin
            if IDs[I] = 0 then
              Len := I
            else
              Len := I+1;
            Break;
          end;
        HIDLanguagesLbl.Caption := Format('HidD_GetIndexedString(0) succeeded (%d languages)', [Len]);
        Break;
      end
      else
      if GetLastError <> ERROR_INVALID_USER_BUFFER then
      begin
        HIDLanguagesLbl.Caption := Format('HidD_GetIndexedString(0) failed (%s)', [SysErrorMessage(GetLastError)]);
        Break;
      end;
    end
  end
  else
    HIDLanguagesLbl.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.HIDStringsBtnClick(Sender: TObject);
var
  I: Integer;
  J: Integer;
  N: Integer;
  Buffer: array [0..256] of WideChar;
begin
  HIDStringsBtn.Tag := 1;
  N := 0;
  if TestDev.OpenFile then
  begin
    Screen.Cursor := crHourGlass;
    for I := 1 to 255 do
    begin
      HIDStringsLbl.Caption :=  Format('testing string %d...', [I]);
      Application.ProcessMessages;
      for J := 0 to SizeOf(Buffer) do
        if HidD_GetIndexedString(TestDev.HidFileHandle, I, Buffer, J) then
        begin
          Inc(N);
          Break;
        end
        else
        if (GetLastError <> ERROR_INVALID_USER_BUFFER) and (GetLastError <> ERROR_CRC) then
        begin
          HIDStringsLbl.Caption := Format('HidD_GetIndexedString(%d) failed with buffer size %d (%s)', [I, J, SysErrorMessage(GetLastError)]);
          Screen.Cursor := crDefault;
          Exit;
        end;
    end;
    Screen.Cursor := crDefault;
    HIDStringsBtn.Tag := 2;
    HIDStringsLbl.Caption :=  Format('%d strings found', [N]);
  end
  else
    HIDStringsLbl.Caption := 'Could not open device file';
end;

procedure TExtendedTestForm.FormShow(Sender: TObject);
var
  Ret: NTSTATUS;
begin
  if PrepData <> nil then
    HidD_FreePreparsedData(PrepData);
  PrepData := nil;
  if TestDev.OpenFile then
    if not HidD_GetPreparsedData(TestDev.HidFileHandle, PrepData) then
      PrepData := nil;

  FillChar(Caps, SizeOf(THIDPCaps), #0);
  CapsOK := False;
  if PrepData <> nil then
  begin
    Ret := HidP_GetCaps(PrepData, Caps);
    CapsOK := Ret = HIDP_STATUS_SUCCESS;
  end;
end;

procedure TExtendedTestForm.FormDestroy(Sender: TObject);
begin
  if PrepData <> nil then
    HidD_FreePreparsedData(PrepData);
end;

end.
